#include "mcmc.h"

namespace NormalTwoVars {

	double sqr(double v){
		return v * v;
	}

	double Exp0(double v){
		if (v > 0.0){
			return 1.0;
		}
		else{
			return Math::Exp(v);
		}
	}


	double MCMC::LogL(double mu1, double mu2, double sgm1, double sgm2, double rho){
		double v = -n * Math::Log(sgm1) - n * Math::Log(sgm2) - 0.5 * n * Math::Log(1.0 - sqr(rho));
		double v1 = (ssumX1 - 2.0 * mu1 * sumX1 + n * sqr(mu1)) / sqr(sgm1)
					- 2.0 * rho * (sumX1X2 - mu1 * sumX2 - mu2 * sumX1 + n * mu1 * mu2) / (sgm1 * sgm2)
					+ (ssumX2 - 2.0 * mu2 * sumX2 + n * sqr(mu2)) / sqr(sgm2);
		v -= v1 / (2.0 * (1.0 - sqr(rho)));
		return v;
	}

	void MCMC::do_mcmc(){
		dmf.Show();
		for (int t = 0; t < NSimu; t++){
			if ((t % 1000) == 0){
				dmf.ShowMessage(" " + t.ToString() + "/" + NSimu.ToString(), " ");
			}

			double y = rn->normalMS(mean1[t], GenMuSgm1);
			double num = LogL(y, mean2[t], sd1[t], sd2[t], rho[t]);
			double den = LogL(mean1[t], mean2[t], sd1[t], sd2[t], rho[t]);
			double a = Exp0(num - den);
			if (rn->uni() < a){
				mean1[t + 1] = y;
				acpt_mean1++;
			}
			else {
				mean1[t + 1] = mean1[t];
			}

			y = rn->normalMS(mean2[t], GenMuSgm2);
			num = LogL(mean1[t + 1], y, sd1[t], sd2[t], rho[t]);
			den = LogL(mean1[t + 1], mean2[t], sd1[t], sd2[t], rho[t]);
			a = Exp0(num - den);
			if (rn->uni() < a){
				mean2[t + 1] = y;
				acpt_mean2++;
			}
			else {
				mean2[t + 1] = mean2[t];
			}

			y = rn->normalMS(sd1[t], GenSDSgm1);
			if (y <= 0.0){
				a = 0.0;
			}
			else {
				num = LogL(mean1[t + 1], mean2[t + 1], y, sd2[t], rho[t]);
				den = LogL(mean1[t + 1], mean2[t + 1], sd1[t], sd2[t], rho[t]);
				a = Exp0(num - den);
			}
			if (rn->uni() < a){
				sd1[t + 1] = y;
				acpt_sd1++;
			}
			else {
				sd1[t + 1] = sd1[t];
			}

			y = rn->normalMS(sd2[t], GenSDSgm2);
			if (y <= 0.0) {
				a = 0.0;
			}
			else {
				num = LogL(mean1[t + 1], mean2[t + 1], sd1[t + 1], y, rho[t]);
				den = LogL(mean1[t + 1], mean2[t + 1], sd1[t + 1], sd2[t], rho[t]);
				a = Exp0(num - den);
			}
			if (rn->uni() < a){
				sd2[t + 1] = y;
				acpt_sd2++;
			}
			else{
				sd2[t + 1] = sd2[t];
			}

			y = rn->normalMS(rho[t], GenRhoSgm);
			if ((y <= -1.0) || (y >= 1.0)) {
				a = 0.0;
			}
			else{
				num = LogL(mean1[t + 1], mean2[t + 1], sd1[t + 1], sd2[t + 1], y);
				den = LogL(mean1[t + 1], mean2[t + 1], sd1[t + 1], sd2[t + 1], rho[t]);
				a = Exp0(num - den);
			}
			if (rn->uni() < a){
				rho[t + 1] = y;
				acpt_rho++;
			}
			else{
				rho[t + 1] = rho[t];
			}
		}
		dmf.Close();
	}

}