#ifndef MCMC_CK
#define MCMC_CK

#include "RNP2to191.h"
#include "DispMessage.h"
#include "common.h"

using namespace rnp2to191;
using namespace NormalTwoVars;
using namespace cmmn;

namespace NormalTwoVars{

	ref class MCMC {
		double ** X;
		int n;
		double sumX1, ssumX1, sumX2, ssumX2, sumX1X2;
		RNP2to191 * rn;

		int NSimu;

	public:
		double * mean1, *mean2, *sd1, * sd2, * rho;
		double GenMuSgm1, GenMuSgm2, GenSDSgm1, GenSDSgm2, GenRhoSgm;
		int acpt_mean1, acpt_mean2, acpt_sd1, acpt_sd2, acpt_rho;

		DispMessage dmf;

		void Clear(){
			if (rn != nullptr) delete rn;
			if (mean1 != nullptr) delete[] mean1;
			if (mean2 != nullptr) delete[] mean2;
			if (sd1 != nullptr) delete[] sd1;
			if (sd2 != nullptr) delete[] sd2;
			if (rho != nullptr) delete[] rho;
		}

		virtual ~MCMC(){
			Clear();
		}

		!MCMC(){
			Clear();
		}

	public:
		void Init(int nJump, int vn, double ** vX, int vNSimu,
			double InitMean1, double InitMean2, double InitSD1, double InitSD2, double InitRho,
			double vGenMuSgm1, double vGenMuSgm2, double vGenSDSgm1, double vGenSDSgm2, double vGenRhoSgm){

			rn = new RNP2to191();
			for (int i = 0; i < nJump; i++)
				rn->Jump2toj(100);

			GenMuSgm1 = vGenMuSgm1;
			GenMuSgm2 = vGenMuSgm2;
			GenSDSgm1 = vGenSDSgm1;
			GenSDSgm2 = vGenSDSgm2;
			GenRhoSgm = vGenRhoSgm;

			X = vX;
			n = vn;
			NSimu = vNSimu;

			mean1 = new double[NSimu + 1];
			mean2 = new double[NSimu + 1];
			sd1 = new double[NSimu + 1];
			sd2 = new double[NSimu + 1];
			rho = new double[NSimu + 1];
			mean1[0] = InitMean1;
			mean2[0] = InitMean2;
			sd1[0] = InitSD1;
			sd2[0] = InitSD2;
			rho[0] = InitRho;

			acpt_mean1 = acpt_mean2 = acpt_sd1 = acpt_sd2 = acpt_rho = 0;

			sumX1 = ssumX1 = sumX2 = ssumX2 = sumX1X2 = 0.0;
			for (int i = 0; i < n; i++){
				sumX1 += X[i][0];
				ssumX1 += sqr(X[i][0]);
				sumX2 += X[i][1];
				ssumX2 += sqr(X[i][1]);
				sumX1X2 += X[i][0] * X[i][1];
			}
		}

		double LogL(double mu1, double mu2, double sgm1, double sgm2, double rho);

		void do_mcmc();

	};

}

#endif
